/**
 * Copyright (c) 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.api.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Options;
import org.junit.Rule;
import org.junit.Test;
import org.junit.contrib.java.lang.system.SystemErrRule;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.ConnectionErrorException;
import org.ow2.petals.cli.api.command.exception.DisconnectionErrorException;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.InvalidConnectionParameterException;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.api.shell.ShellExtension;
import org.ow2.petals.cli.api.shell.exception.ConnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DisconnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DuplicatedCommandException;
import org.ow2.petals.cli.api.shell.exception.ShellException;

import jline.console.completer.Completer;

public class CommandRegistrationTestCase {

    @Rule
    public final SystemErrRule systemErr = new SystemErrRule().muteForSuccessfulTests().enableLog();

    private class CommandRegistratorShell implements Shell {

        private final Map<String, Command> commands = new HashMap<String, Command>();

        private ConnectionCommand connectionCommand = null;

        private DisconnectionCommand disconnectionCommand = null;

        @Override
        public void registersCommand(final Command command) throws DuplicatedCommandException, IllegalArgumentException {
            if (this.commands.put(command.getName(), command) != null) {
                // The command is already registered
                throw new DuplicatedCommandException(command.getName());
            }
        }

        @Override
        public Map<String, Command> getCommands() {
            return this.commands;
        }

        @Override
        public void registersConnectionCommand(final ConnectionCommand connectionCommand)
                throws ConnectionCommandAlreadyRegisteredException, DuplicatedCommandException,
                IllegalArgumentException {
            if (this.connectionCommand == null) {
                this.connectionCommand = connectionCommand;
                this.registersCommand(connectionCommand);
            } else {
                throw new ConnectionCommandAlreadyRegisteredException(this.connectionCommand.getName());
            }
        }

        @Override
        public void registersDisconnectionCommand(final DisconnectionCommand disconnectionCommand)
                throws DisconnectionCommandAlreadyRegisteredException, DuplicatedCommandException,
                IllegalArgumentException {
            if (this.disconnectionCommand == null) {
                this.disconnectionCommand = disconnectionCommand;
                this.registersCommand(disconnectionCommand);
            } else {
                throw new DisconnectionCommandAlreadyRegisteredException(this.disconnectionCommand.getName());
            }
        }

        @Override
        public void exit(int status) {

        }

        @Override
        public void setExitStatus(int exitStatus) {

        }

        @Override
        public PrintStream getPrintStream() {
            return null;
        }

        @Override
        public String interpolate(String str) {
            return null;
        }

        @Override
        public PrintStream getErrorStream() {
            return null;
        }

        @Override
        public String askQuestion(final String question, final boolean isReplyPassword) throws ShellException {
            return null;
        }

        @Override
        public boolean confirms(final String message) throws ShellException {
            return false;
        }

        @Override
        public boolean isInteractive() {
            return false;
        }

        @Override
        public void setPrompt(final String prompt) {
        }

        @Override
        public boolean isDebugModeEnable() {
            return false;
        }

        @Override
        public void setConnectionParameters(final ConnectionParameters connectionParameters) {
        }

        @Override
        public ConnectionParameters getConnectionParameters() {
            return null;
        }

        @Override
        public boolean isAsynchronousCommandInProgress() {
            return false;
        }

        @Override
        public void addAsynchronousCommand() {
        }

        @Override
        public void removeAsynchronousCommand() {
        }

        @Override
        public ConnectionCommand getConnectionCommand() {
            return this.connectionCommand;
        }

        @Override
        public DisconnectionCommand getDisconnectionCommand() {
            return this.disconnectionCommand;
        }

        @Override
        public ShellExtension[] getExtensions() {
            return null;
        }

        @Override
        public void setDefaultPrompt() {
            // NOP
        }

        @Override
        public Preferences getPreferences() {
            return null;
        }

        @Override
        public void onConnectionEstablished() {
            // NOP
        }

        @Override
        public void onDisconnection() {
            // NOP
        }

        @Override
        public boolean isConnectionEstablished() {
            // TODO Auto-generated method stub
            return false;
        }
    }

    /**
     * <p>
     * Check the command registration:
     * <ul>
     * <li>first, with a valid command,</li>
     * <li>2nd, trying to register again the command instance,</li>
     * <li>3rd, trying to register another connection command instance.</li>
     * <li>4th, trying to register another disconnection command instance.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>the first registration succeeds,</li>
     * <li>the 2nd registration fails with an error message on stderr,</li>
     * <li>the 3rd registration fails with an error message on stderr,</li>
     * <li>the 4th registration fails with an error message on stderr,</li>
     * </ul>
     * </p>
     */
    @Test
    public void testRegisterCommands() throws DuplicatedCommandException {

        final CommandRegistratorShell crs = new CommandRegistratorShell();

        // First registration
        CommandRegistration.registers(crs);
        final Command nullCommand = new NullCommand();

        final Map<String, Command> commands = crs.getCommands();
        assertEquals("Too much commands registered", 3, commands.size());
        assertEquals("Unexpected registered command", nullCommand.getName(), commands.get(nullCommand.getName())
                .getName());

        // 2nd registration

        CommandRegistration.registers(crs);

        final String error = systemErr.getLog();
        assertFalse("No error message print", error.isEmpty());
        assertTrue("The command name is not included in the error message.",
                error.contains("'" + nullCommand.getName() + "'"));

        // 3rd registration
        try {
            crs.registersConnectionCommand(new ConnectionCommand() {

                @Override
                public void setShell(final Shell shell) {
                }

                @Override
                public void resetOptions() {
                }

                @Override
                public boolean isConnectionRequired() {
                    return false;
                }

                @Override
                public String getUsage() {
                    return null;
                }

                @Override
                public Shell getShell() {
                    return null;
                }

                @Override
                public String getOptionsDescription() {
                    return null;
                }

                @Override
                public Options getOptions() {
                    return null;
                }

                @Override
                public Map<String, Completer> getOptionCompleters() {
                    return null;
                }

                @Override
                public String getName() {
                    return "another-connection-command";
                }

                @Override
                public String getDescription() {
                    return null;
                }

                @Override
                public Completer getDefaultCompleter() {
                    return null;
                }

                @Override
                public void execute(final String[] args) throws CommandException {
                }

                @Override
                public AuthenticatedConnectionParameters connect(final ConnectionParameters connectionParameters,
                        final boolean isYesFlagEnabledAtCmdLevel) throws ConnectionErrorException, CommandException {
                    return null;
                }

                @Override
                public ConnectionParameters parseConnectionParameters(final CommandLine cmd,
                        final boolean useDefaultConnection) throws CommandException, PreferenceFileException,
                        InvalidConnectionParameterException {
                    return null;
                }
            });

            fail("Another connection command registration succeeds !");
        } catch (final ConnectionCommandAlreadyRegisteredException e) {
            // It's the expected exception
            assertEquals(NullConnectionCommand.NAME, e.getCommandName());
        }

        // 4th registration
        try {
            crs.registersDisconnectionCommand(new DisconnectionCommand() {

                @Override
                public Options getOptions() {
                    return null;
                }

                @Override
                public void resetOptions() {
                    // NOP
                }

                @Override
                public String getOptionsDescription() {
                    return null;
                }

                @Override
                public String getName() {
                    return "another-disconnection-command";
                }

                @Override
                public String getUsage() {
                    return null;
                }

                @Override
                public String getDescription() {
                    return null;
                }

                @Override
                public Completer getDefaultCompleter() {
                    return null;
                }

                @Override
                public Map<String, Completer> getOptionCompleters() {
                    return null;
                }

                @Override
                public Shell getShell() {
                    return null;
                }

                @Override
                public void setShell(final Shell shell) {
                    // NOP
                }

                @Override
                public void execute(final String[] args) throws CommandException {
                    // NOP
                }

                @Override
                public boolean isConnectionRequired() {
                    return false;
                }

                @Override
                public void disconnect() throws DisconnectionErrorException, CommandException {
                    // NOP
                }

                @Override
                public boolean isConnected() throws DisconnectionErrorException {
                    return false;
                }
            });

            fail("Another disconnection command registration succeeds !");
        } catch (final DisconnectionCommandAlreadyRegisteredException e) {
            // It's the expected exception
            assertEquals(NullDisconnectionCommand.NAME, e.getCommandName());
        }
    }

}
